@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # Copyright (c) 2010 Mark Sandstrom
-# Copyright (c) 2011-2013 Raphaël Barrois
+# Copyright (c) 2011-2015 Raphaël Barrois
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -22,7 +22,6 @@
 
 
 import itertools
-import warnings
 import logging
 
 from . import compat
@@ -58,6 +57,23 @@ class OrderedDeclaration(object):
         raise NotImplementedError('This is an abstract method')
 
 
+class LazyFunction(OrderedDeclaration):
+    """Simplest OrderedDeclaration computed by calling the given function.
+
+    Attributes:
+        function (function): a function without arguments and
+            returning the computed value.
+    """
+
+    def __init__(self, function, *args, **kwargs):
+        super(LazyFunction, self).__init__(*args, **kwargs)
+        self.function = function
+
+    def evaluate(self, sequence, obj, create, extra=None, containers=()):
+        logger.debug("LazyFunction: Evaluating %r on %r", self.function, obj)
+        return self.function()
+
+
 class LazyAttribute(OrderedDeclaration):
     """Specific OrderedDeclaration computed using a lambda.
 
@@ -161,12 +177,19 @@ class Iterator(OrderedDeclaration):
     def __init__(self, iterator, cycle=True, getter=None):
         super(Iterator, self).__init__()
         self.getter = getter
+        self.iterator = None
 
         if cycle:
-            iterator = itertools.cycle(iterator)
-        self.iterator = utils.ResetableIterator(iterator)
+            self.iterator_builder = lambda: utils.ResetableIterator(itertools.cycle(iterator))
+        else:
+            self.iterator_builder = lambda: utils.ResetableIterator(iterator)
 
     def evaluate(self, sequence, obj, create, extra=None, containers=()):
+        # Begin unrolling as late as possible.
+        # This helps with ResetableIterator(MyModel.objects.all())
+        if self.iterator is None:
+            self.iterator = self.iterator_builder()
+
         logger.debug("Iterator: Fetching next value from %r", self.iterator)
         value = next(iter(self.iterator))
         if self.getter is None:
@@ -195,7 +218,7 @@ class Sequence(OrderedDeclaration):
         self.type = type
 
     def evaluate(self, sequence, obj, create, extra=None, containers=()):
-        logger.debug("Sequence: Computing next value of %r for seq=%d", self.function, sequence)
+        logger.debug("Sequence: Computing next value of %r for seq=%s", self.function, sequence)
         return self.function(self.type(sequence))
 
 
@@ -209,7 +232,7 @@ class LazyAttributeSequence(Sequence):
             of counter for the 'function' attribute.
     """
     def evaluate(self, sequence, obj, create, extra=None, containers=()):
-        logger.debug("LazyAttributeSequence: Computing next value of %r for seq=%d, obj=%r",
+        logger.debug("LazyAttributeSequence: Computing next value of %r for seq=%s, obj=%r",
                 self.function, sequence, obj)
         return self.function(obj, self.type(sequence))
 
@@ -417,6 +440,55 @@ class List(SubFactory):
             **params)
 
 
+# Parameters
+# ==========
+
+
+class ComplexParameter(object):
+    """A complex parameter, to be used in a Factory.Params section.
+
+    Must implement:
+    - A "compute" function, performing the actual declaration override
+    - Optionally, a get_revdeps() function (to compute other parameters it may alter)
+    """
+
+    def compute(self, field_name, declarations):
+        """Compute the overrides for this parameter.
+
+        Args:
+        - field_name (str): the field this parameter is installed at
+        - declarations (dict): the global factory declarations
+
+        Returns:
+            dict: the declarations to override
+        """
+        raise NotImplementedError()
+
+    def get_revdeps(self, parameters):
+        """Retrieve the list of other parameters modified by this one."""
+        return []
+
+
+class Trait(ComplexParameter):
+    """The simplest complex parameter, it enables a bunch of new declarations based on a boolean flag."""
+    def __init__(self, **overrides):
+        self.overrides = overrides
+
+    def compute(self, field_name, declarations):
+        if declarations.get(field_name):
+            return self.overrides
+        else:
+            return {}
+
+    def get_revdeps(self, parameters):
+        """This might alter fields it's injecting."""
+        return [param for param in parameters if param in self.overrides]
+
+
+# Post-generation
+# ===============
+
+
 class ExtractionContext(object):
     """Private class holding all required context from extraction to postgen."""
     def __init__(self, value=None, did_extract=False, extra=None, for_field=''):
@@ -502,14 +574,6 @@ class RelatedFactory(PostGenerationDeclaration):
 
     def __init__(self, factory, factory_related_name='', **defaults):
         super(RelatedFactory, self).__init__()
-        if factory_related_name == '' and defaults.get('name') is not None:
-            warnings.warn(
-                "Usage of RelatedFactory(SomeFactory, name='foo') is deprecated"
-                " and will be removed in the future. Please use the"
-                " RelatedFactory(SomeFactory, 'foo') or"
-                " RelatedFactory(SomeFactory, factory_related_name='foo')"
-                " syntax instead", PendingDeprecationWarning, 2)
-            factory_related_name = defaults.pop('name')
 
         self.name = factory_related_name
         self.defaults = defaults
